﻿using System;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using KcpNet;

namespace KcpClientTest
{
    class Program
    {
        static void Main(string[] args)
        {
            var client = new KcpClient();
            Task.Run(async () =>
            {
                client.Connect(new IPEndPoint(IPAddress.Parse("127.0.0.1"), 6000), 5000);
                client.Connection.PingRequested += (sender, eventArgs) =>
                {
                    Console.WriteLine("Ping requested.");
                };
                client.Connection.PingResponded += (sender, eventArgs) =>
                {
                    Console.WriteLine("Ping respond.");
                };
                await SendStringAsync(client, $"哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈哈").ConfigureAwait(false);
                var i = 0;
                while (true)
                {
                    i++;
                    try
                    {

                        var result = await ReadStringAsync(client).ConfigureAwait(false);
                        Console.WriteLine($"Client [{client.Connection.Id}] received: " + result);
                    }
                    catch (Exception ex)
                    {
                        client.Close();
                        Console.WriteLine(ex.Message + " Connection closed.");
                        break;
                    }
                }
            });
            Console.Read();
        }


        private static async Task SendStringAsync(KcpClient client, string str)
        {
            var strData = Encoding.UTF8.GetBytes(str);
            var lengthData = BitConverter.GetBytes(strData.Length);
            await client.SendAsync(lengthData).ConfigureAwait(false);
            await client.SendAsync(strData).ConfigureAwait(false);
        }

        private static async Task<string> ReadStringAsync(KcpClient client)
        {
            var lengthData = await ReadBytesAsync(client, 4).ConfigureAwait(false);
            var length = BitConverter.ToInt32(lengthData);
            var strData = await ReadBytesAsync(client, length);
            return Encoding.UTF8.GetString(strData);
        }

        private static async Task<byte[]> ReadBytesAsync(KcpClient client, int size)
        {
            if (size > 0)
            {
                var bytes = new byte[size];
                var length = size;
                var startIndex = 0;
                while (length > 0)
                {
                    var buffer = new byte[length];
                    var bytesRead = await client.ReadAsync(buffer).ConfigureAwait(false);
                    if (bytesRead == 0)
                    {
                        throw new InvalidOperationException("Connection closed.");
                    }
                    if (bytesRead > 0)
                    {
                        Array.Copy(buffer, 0, bytes, startIndex, bytesRead);
                        startIndex += bytesRead;
                        length -= bytesRead;
                    }
                }

                return bytes;
            }

            return null;
        }
    }
}
