﻿using System;
using System.IO;
using System.Text;
using System.Threading.Tasks;

namespace KcpNet
{
    public class KcpNetBase
    {
        protected static readonly byte[] ProtocolHeader = Encoding.ASCII.GetBytes("KCP");

        protected async Task<byte[]> ReadBytesAsync(Stream stream, int count)
        {
            var result = new byte[count];
            var bytesRead = 0;
            while (count > 0)
            {
                var readCount = await stream.ReadAsync(result, bytesRead, count).ConfigureAwait(false);
                if (readCount == 0)
                {
                    throw new InvalidOperationException("Stream is empty");
                }
                bytesRead += readCount;
                count -= readCount;
            }

            if (bytesRead != result.Length)
            {
                throw new EndOfStreamException();
            }
            return result;
        }

        protected async Task<byte> ReadByteAsync(Stream stream)
        {
            var bytes = await ReadBytesAsync(stream, 1).ConfigureAwait(false);
            return bytes[0];
        }

        protected async Task<ushort> ReadUInt16Async(Stream stream)
        {
            var bytes = await ReadBytesAsync(stream, 2).ConfigureAwait(false);
            return BitConverter.ToUInt16(bytes);
        }

        protected async Task<uint> ReadUInt32Async(Stream stream)
        {
            var bytes = await ReadBytesAsync(stream, 4).ConfigureAwait(false);
            return BitConverter.ToUInt32(bytes);
        }

    }
}
