﻿using System;
using System.Net;
using System.Net.Sockets;

namespace KcpNet
{
    public class KcpUdpIo : IKcpIo
    {
        private readonly UdpClient _client;

        private readonly IPEndPoint _remoteEndPoint;

        public event EventHandler<byte[]> Received;

        public KcpUdpIo(UdpClient client, IPEndPoint remoteEndPoint)
        {
            _client = client;
            _remoteEndPoint = remoteEndPoint;
        }

        public void Output(byte[] buffer)
        {
            var sentSize = _client.Send(buffer, buffer.Length, _remoteEndPoint);
            if (sentSize != buffer.Length)
            {
                throw new InvalidOperationException("Send buffer failed.");
            }
        }

        public void Output(Memory<byte> memory, int size)
        {
            var buffer = memory.Slice(0, size).ToArray();
            var sentSize = _client.Send(buffer, buffer.Length, _remoteEndPoint);
            if (sentSize != buffer.Length)
            {
                throw new InvalidOperationException("Send buffer failed.");
            }
        }

        public void Input(byte[] buffer)
        {
            Received?.Invoke(this, buffer);
        }


        public void Input(Memory<byte> memory, int size)
        {
            var buffer = memory.Slice(0, size).ToArray();
            Received?.Invoke(this, buffer);
        }
    }
}
