﻿using System;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using KcpNet;

namespace KcpServerTest
{
    class Program
    {
        static void Main(string[] args)
        {
            var server = new KcpListener(new IPEndPoint(IPAddress.Any, 6000));
            server.Start();
            Task.Run(async () =>
            {
                while (true)
                {
                    var connection = await server.GetConnectionAsync().ConfigureAwait(false);
                    if (connection != null)
                    {
                        HandleServerConnection(connection);
                    }
                    else
                    {
                        break;
                    }
                }
            });
            Console.Read();
        }

        private static async Task<byte[]> ReadServerBytesAsync(KcpConnection connection, int size)
        {
            var bytes = new byte[size];
            var length = size;
            var startIndex = 0;
            while (length > 0)
            {
                var buffer = new byte[length];
                var bytesRead = await connection.ReadAsync(buffer).ConfigureAwait(false);
                if (bytesRead == 0)
                {
                    throw new InvalidOperationException("Connection could be closed.");
                }
                Array.Copy(buffer, 0, bytes, startIndex, bytesRead);
                startIndex += bytesRead;
                length -= bytesRead;
            }

            return bytes;
        }


        private static void HandleServerConnection(KcpConnection connection)
        {
            Task.Run(async () =>
            {
                try
                {

                    while (true)
                    {
                        var lengthData = await ReadServerBytesAsync(connection, 4);
                        var length = BitConverter.ToInt32(lengthData, 0);
                        var data = await ReadServerBytesAsync(connection, length).ConfigureAwait(false);
                        Console.WriteLine($"Server connection [{connection.Id}] received: " + Encoding.UTF8.GetString(data));
                        while (true)
                        {
                            var replyData = Encoding.UTF8.GetBytes("OK");
                            await connection.SendAsync(BitConverter.GetBytes(replyData.Length)).ConfigureAwait(false);
                            await connection.SendAsync(replyData).ConfigureAwait(false);
                        }
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            });

        }
    }
}
